@@ -1,8 +1,8 @@
 #########################################################################
-# - Net::UCP 0.40 - 
+# - Net::UCP 0.42 - 
 # 
-# Version : 0.40
-# Date    : 29/05/2009
+# Version : 0.42
+# Date    : 16/10/2011
 #
 # Library based on EMI - UCP INTERFACE Specification 
 # Version 3.5 of December 1999 
@@ -37,7 +37,7 @@ use vars qw(@ISA @EXPORT @EXPORT_OK $VERSION);
 @EXPORT = qw();
 @EXPORT_OK = ();
 
-$VERSION = '0.40';
+$VERSION = '0.42';
 
 $VERSION = eval $VERSION; 
 
@@ -50,6 +50,7 @@ sub new { bless({}, shift())->_init(@_); }
 # login to SMSC
 sub login {
     my $self = shift();
+	    
     my %args = (
 		SMSC_ID    => '',
 		SMSC_PW    => '',
@@ -208,9 +209,7 @@ sub send_sms {
 
 sub send_sms_multipart {
     my $self = shift;
-    
-    my %argsLong =(
-		   RECIPIENT      => '',
+    my %argsLong =(RECIPIENT      => '',
 		   MESSAGE_TEXT   => '',
 		   SENDER_TEXT    => '',
 		   UDH            => undef,
@@ -219,45 +218,20 @@ sub send_sms_multipart {
 		   TIMEOUT        => undef,
 		   @_);
 
-    my $chunkBlock = 134;                
     my $messageId  = random_int(1,255);  
-    
     my $multiHash;
-    
-    my $txtTmp    = $argsLong{MESSAGE_TEXT};
     my $countPart = 0;
-    my $chunk;
     
-    while (length($txtTmp) != 0) {
-	
-        if (length($txtTmp) > 134) {
-            $txtTmp =~ s/((.){134})//;
-            $chunk = $1;
-        } else {
-            $chunk  = $txtTmp;
-            $txtTmp = '';
-        }
-	
+    foreach my $chunk ($argsLong{MESSAGE_TEXT}=~m/(.{1,134})/sg){
         $multiHash->{++$countPart} = $chunk;
-	
     }
-    
-    my $textChunk = '';
-    my $k        = '';
-    my $udhChunk = '';
-    my $ret;
-
-    while (($k, $textChunk) = each %{$multiHash}) {
-        $udhChunk = "0106050003" 
-	    . sprintf("%02X", $messageId)
-            . sprintf("%02X", $countPart)
-            . sprintf("%02X", $k);
-	
-        $ret = $self->send_sms(
+    while (my ($k, $textChunk) = each %{$multiHash}) {
+        my $udh=sprintf('%s%02X%02X%02X',"0106050003",$messageId,$countPart,$k);
+        my $ret = $self->send_sms(
 			       RECIPIENT      => $argsLong{RECIPIENT},
 			       MESSAGE_TEXT   => $textChunk,
 			       SENDER_TEXT    => $argsLong{SENDER_TEXT},
-			       UDH            => $udhChunk,
+			       UDH            => $udh
 			       );
     
 	return $ret if (!$ret || !defined($ret)); #it could be better
@@ -409,6 +383,8 @@ sub _init {
 	
 	$self->{SRC_HOST} = $args{SRC_HOST};
 	$self->{SRC_PORT} = $args{SRC_PORT};
+        # itguru fix
+	$self->{SHORT_CODE} = defined($args{SHORT_CODE})&&length($args{SHORT_CODE})?$args{SHORT_CODE}:undef;
 	
 	$self->{SOCK} = undef;
 	
@@ -453,7 +429,7 @@ sub wait_in_loop {
 	if ($retval) {
 	    my ($buffer,$response);
 	    do {
-                read($socket,$buffer,1);
+                sysread($socket,$buffer,1);
                 $response.=$buffer;
             } until ($buffer eq ETX);   
 	    
@@ -515,7 +491,7 @@ sub parse_message {
  
     my $ref_mess = undef;
     
-    if (my($optype) = $resp =~ m{^\d{2}/\d{5}/.*?/(01|02|03|30|51|52|53|54|55|56|57|58|60|61)/.*}) {
+    if (my($optype) = $resp =~ m{^\d{2}/\d{5}/.*?/(01|02|03|30|31|51|52|53|54|55|56|57|58|60|61)/.*}) {
 	my $parse_method = "parse_$optype";
 	$ref_mess = $self->$parse_method($resp);
     }
@@ -1346,7 +1322,7 @@ sub _make_5x {
 	    ? $self->{OBJ_EMI_COMMON}->encode_7bit($arg->{oadc}) 
 	    : $arg->{oadc} 
 	if exists $arg->{otoa} ;
-	    
+
 	my $string = 
 	    (exists $arg->{adc} ? $arg->{adc} : '') .
 	    UCP_DELIMITER .
@@ -1975,7 +1951,7 @@ sub transmit_msg {
 		    $rd=undef;
 		    local($SIG{ALRM})=sub{die("alarm\n")}; # NB: \n required
 		    alarm($timeout);
-		    $rd=read($self->{SOCK},$buffer,1);
+		    $rd=sysread($self->{SOCK},$buffer,1);
 		    alarm(0);
 		};
 		# Propagate unexpected errors.
@@ -1983,7 +1959,7 @@ sub transmit_msg {
 	    }
 	    else {
 		# No alarm() implemented. Must do a (potentially) blocking call to read().
-		$rd=read($self->{SOCK},$buffer,1);
+		$rd=sysread($self->{SOCK},$buffer,1);
 	    }
 	    defined($rd)||do{ # undef, read error.
 		$errtxt="Failed to read from SMSC socket. Never received ETX. Remote end closed?";
@@ -2821,7 +2797,7 @@ Marco Romano, E<lt>nemux@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2004-2008 by Marco Romano
+Copyright (C) 2004-2011 by Marco Romano
 
   This library is free software; you can redistribute it and/or modify
   it under the same terms as Perl itself, either Perl version 5.8.4 or,
